/* Includes ------------------------------------------------------------------*/
#include "stm32f1xx_hal.h"
#include "font6x8.h"
#include "spi.h"
#include "LCD_Nokia.h"

/* Private variables ---------------------------------------------------------*/
uint8_t lcd8544_buff[84*6]; // LCD buffer
//=============
unsigned char y, x, _x, _y, _mode;
unsigned int adr;
unsigned char data;
unsigned char lcd_YP;
unsigned char lcd_YC;
//==============

/* Private function prototypes -----------------------------------------------*/

/* -----------------------------------------------*/
void lcd8544_putpix(unsigned char x, unsigned char y, unsigned char mode) {

	if ((x>83) || (y>47)) return;

	adr=(y>>3)*84+x;
	data=(1<<(y&0x07));

	if (mode) 
		lcd8544_buff[adr]|=data;
   else 
		 lcd8544_buff[adr]&=~data;
}
void lcd8544_line(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char mode)
{
signed char   dx, dy, sx, sy;
unsigned char  x,  y, mdx, mdy, l;


  dx=x2-x1; dy=y2-y1;

  if (dx>=0) { mdx=dx; sx=1; } else { mdx=x1-x2; sx=-1; }
  if (dy>=0) { mdy=dy; sy=1; } else { mdy=y1-y2; sy=-1; }

  x=x1; y=y1;

  if (mdx>=mdy) {
     l=mdx;
     while (l>0) {
         if (dy>0) { y=y1+mdy*(x-x1)/mdx; }
            else { y=y1-mdy*(x-x1)/mdx; }
         lcd8544_putpix(x,y,mode);
         x=x+sx;
         l--;
     }
  } else {
     l=mdy;
     while (l>0) {
        if (dy>0) { x=x1+((mdx*(y-y1))/mdy); }
          else { x=x1+((mdx*(y1-y))/mdy); }
        lcd8544_putpix(x,y,mode);
        y=y+sy;
        l--;
     }
  }
  lcd8544_putpix(x2, y2, mode);

}

void lcd8544_rect(unsigned char x1, unsigned char y1, unsigned char x2, unsigned char y2, unsigned char mode) {
	lcd8544_line(x1,y1, x2,y1, mode);
	lcd8544_line(x1,y2, x2,y2, mode);
	lcd8544_line(x1,y1, x1,y2, mode);
	lcd8544_line(x2,y1, x2,y2, mode);
}

void lcd8544_putchar(unsigned char px, unsigned char py, unsigned char ch, unsigned char mode) {
	const unsigned char *fontpointer;
	unsigned char temp;

	if (ch<127) {	//  
		fontpointer=NewFontLAT; ch=ch-32; }
		else	{	//   
		fontpointer=NewFontRUS; ch=ch-192;}

    unsigned char lcd_YP=7- (py & 0x07);    //     
    unsigned char lcd_YC=(py & 0xF8)>>3; 	//     
    unsigned char x;
	for (x=0; x<6; x++) {

		temp=*(fontpointer+ch*6+x);

		if (mode!=0) {
			temp=~temp;
			if (py>0) lcd8544_putpix(px, py-1, 1);	//      -   
		}

		temp&=0x7F;
		lcd8544_buff[lcd_YC*84+px] &= (0xFF>>(lcd_YP+1));
		lcd8544_buff[lcd_YC*84+px] |= (temp<<(7-lcd_YP)); 	//    
//		lcd8544_buff[lcd_YC*84+px] = (temp<<(7-lcd_YP)); 	//    

		lcd8544_buff[(lcd_YC+1)*84+px] &= (0xFF<<(7-lcd_YP));
	    if (lcd_YP<7) lcd8544_buff[(lcd_YC+1)*84+px] |= (temp>>(lcd_YP+1)); 	//    
//	    if (lcd_YP<7) lcd8544_buff[(lcd_YC+1)*84+px] = (temp>>(lcd_YP+1)); 	//    

		px++;
		if (px>83) return;
	}
}

void lcd8544_leftchline(unsigned char x, unsigned char y) {
	if (x>0) lcd8544_line(x-1, y-1, x-1, y+6, 1);
}

void lcd8544_putstr(unsigned char x, unsigned char y, const unsigned char str[], unsigned char mode) {
	if (mode) lcd8544_leftchline(x, y);
	while (*str!=0) {
		lcd8544_putchar(x, y, *str, mode);
		x=x+6;
		str++;
	}
}


void LCD_Init(void)
{
	HAL_Delay(1);
	HAL_GPIO_WritePin(LCD_RST_GPIO_Port, LCD_RST_Pin, GPIO_PIN_SET);
	Clear_LCD_buffer();
	LCD_send_cmd(0x21); // ON Extended cdm set
	LCD_send_cmd(0x13); // set Bias System
	LCD_send_cmd(0x04); // set Temperature Coefficient
	LCD_send_cmd(0xC1); // write VOP to register
	LCD_send_cmd(0x20); // ON Normal cdm set
	LCD_send_cmd(0x0C); // set normal mode
	lcd8544_buff[230] = 0xff;
	lcd8544_buff[231] = 0xff;
	lcd8544_buff[232] = 0xff;
	lcd8544_buff[233] = 0xff;
	LCD_refresh();
}

void LCD_send_cmd(uint8_t data)
{
	HAL_GPIO_WritePin(GPIOA, LCD_DC_Pin|LCD_CE_Pin, GPIO_PIN_RESET);
	HAL_SPI_Transmit(&hspi1, &data, 1, 10);
	HAL_GPIO_WritePin(GPIOA, LCD_DC_Pin|LCD_CE_Pin, GPIO_PIN_SET);
}

void LCD_refresh(void)
{
	LCD_send_cmd(0x40); // set X=0
	LCD_send_cmd(0x80); // set Y=0
	HAL_GPIO_WritePin(GPIOA, LCD_CE_Pin, GPIO_PIN_RESET); // CE=0
	HAL_SPI_Transmit(&hspi1, lcd8544_buff, 504, 10); // Send LCD buffer over SPI
	HAL_GPIO_WritePin(GPIOA, LCD_CE_Pin, GPIO_PIN_SET); // CE=1
}

void Clear_LCD_buffer(void)
{
		uint32_t i;
		for(i=0;i<504;i++) lcd8544_buff[i] = 0;
}
