/**
  ******************************************************************************
  * File Name          : freertos.c
  * Description        : Code for freertos applications
  ******************************************************************************
  *
  * COPYRIGHT(c) 2016 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */

/* Includes ------------------------------------------------------------------*/
#include "FreeRTOS.h"
#include "task.h"
#include "cmsis_os.h"
#include "rtc.h"
#include "LCD_Nokia.h"
#include "usart.h"
#include "ds18b20.h"
#include "ff.h"
#include "fatfs.h"
#include "string.h"

/* defines state_disk */
#define STOP	0
#define START	1
#define LOGGING	2
#define CREATE_OK	3
#define CREATE_ERR	4
#define SAVED_OK	5
#define SAVED_ERR	6

/* Variables -----------------------------------------------------------------*/
osThreadId defaultTaskHandle;
osThreadId DisplayHandle;
osThreadId USARTHandle;


/* USER CODE BEGIN Variables */
RTC_TimeTypeDef sTime;
RTC_DateTypeDef sDate;
extern uint8_t transmitBuffer[10];
extern uint32_t RTC_ReadTimeCounter(RTC_HandleTypeDef* hrtc);

extern uint8_t lcd8544_buff[84*6]; // LCD buffer
extern unsigned char _x, _y, _mode;
int16_t temperature, av_temperature=0;
uint32_t period = 10; // 
uint32_t period_complete = 0; 
uint8_t	state_disk = STOP;
 
FRESULT res;

/* USER CODE END Variables */

/* Function prototypes -------------------------------------------------------*/
void StartDefaultTask(void const * argument);
void StartUSART(void const * argument);

extern void MX_FATFS_Init(void);
void MX_FREERTOS_Init(void); /* (MISRA C 2004 rule 8.1) */

/* Init FreeRTOS */

void MX_FREERTOS_Init(void) {
 
  /* Create the thread(s) */
  /* definition and creation of defaultTask */
  osThreadDef(defaultTask, StartDefaultTask, osPriorityNormal, 0, 512);
  defaultTaskHandle = osThreadCreate(osThread(defaultTask), NULL);

  /* definition and creation of USART */
  osThreadDef(USART, StartUSART, osPriorityIdle, 0, 128);
  USARTHandle = osThreadCreate(osThread(USART), NULL);

  /* USER CODE BEGIN RTOS_THREADS */
  /* add threads, ... */
  /* USER CODE END RTOS_THREADS */

  /* USER CODE BEGIN RTOS_QUEUES */
  /* add queues, ... */
  /* USER CODE END RTOS_QUEUES */
}
void vApplicationIdleHook( void )
{
	HAL_PWR_EnterSLEEPMode(0,PWR_SLEEPENTRY_WFI);
}
void DrawGraphGrid(void)
{
	Clear_LCD_buffer();	
	/*        */
	lcd8544_putstr(0,11,(uint8_t *)"40",0);
	lcd8544_line(13,14,83,14,1);
	lcd8544_putstr(0,21,(uint8_t *)"20",0);
	lcd8544_line(13,24,83,24,1);
	lcd8544_putchar(0,31,'0',0);
	lcd8544_line(13,34,83,34,1);
}

void MoveGraphLeft()
{
	uint16_t i,j;
	/*     x1=14, y1=8, x2=83, y2=40 */
	for(j=84;j<420;j=j+84){ //   
		for(i=13; i<83;i++){ //   X
			lcd8544_buff[j+i]=lcd8544_buff[j+i+1];
		}
		lcd8544_buff[j+i]=0;
	}
	/*    */
	lcd8544_putpix(83,14,1);
	lcd8544_putpix(83,24,1);
	lcd8544_putpix(83,34,1);
	lcd8544_putpix(83,39,1);
}

void Display(void)
{
	osDelay(900);

	__disable_irq (); //       
	temperature = ds_read_temp(); //  
	__enable_irq (); //   
	
	av_temperature = (av_temperature + temperature)/2; //  
	
	_mode=1; _x=0; _y=40; //     LCD
	printf("%02d:%02d:%02d  %3dC", sTime.Hours, sTime.Minutes, sTime.Seconds, temperature>>4); //     

	_mode=1; _x=0; _y=1; //   
	if (state_disk) 
	{
		switch (state_disk){
			case CREATE_OK: 
				printf("  F_CREATE OK "); //     
				state_disk = LOGGING;
				break;
			case CREATE_ERR:
				printf("F_CREATE ERR%d", res); //     
				state_disk = STOP;
				break;
			case SAVED_OK:
				MoveGraphLeft();
				lcd8544_line(83,(68-(av_temperature>>4))/2,83,1+(68-(av_temperature>>4))/2,1); //    
				printf("SAVED %02d:%02d:%02d", sTime.Hours, sTime.Minutes, sTime.Seconds); //     
				av_temperature = temperature; //     
				state_disk = LOGGING;
				break;
			case SAVED_ERR:
				printf("DISK ERR %d   _\n\r", res); //      
				state_disk = STOP;
				break;
		}
	}
	else
	{
		printf(" STOP LOGGING "); //     
		period_complete = HAL_GetTick(); //   
	}
	LCD_refresh(); //  
}

/* StartDefaultTask function */
void StartDefaultTask(void const * argument)
{
	FATFS fileSystem;
  FIL LogFile;
	char path[13] = "xxxxxxxx.LOG\0";

	__disable_irq (); //       
	ds_read_temp(); //   
	__enable_irq (); //   
	DrawGraphGrid(); //        

  /* Infinite loop */
  for(;;)
  {
		HAL_RTC_GetTime(&hrtc, &sTime, RTC_FORMAT_BIN);
		HAL_RTC_GetDate(&hrtc, &sDate, RTC_FORMAT_BIN);

		if(state_disk == START) //****    ****
		{
			DrawGraphGrid(); //       
			_mode=1; _x=0; _y=1; //   
			printf("START\n\r"); //     
			LCD_refresh(); //  
			MX_FATFS_Init(); //  FATFS
		
			_mode=1; _x=0; _y=1; //   

			res=f_mount(&fileSystem, USER_Path, 1); //    
			if(res == FR_OK) 
			{
				sprintf(path, "%08x.LOG", RTC_ReadTimeCounter(&hrtc)+((sDate.Year*365+sDate.Month*31+sDate.Date)<<16)); //   
				res = f_open(&LogFile, (char*)path, FA_WRITE | FA_CREATE_ALWAYS); //    
				f_printf(&LogFile, "DD:MM:YY hh:mm:ss\ttC\n\n"); //     
				res = f_close(&LogFile); //  
			}
			if (res == FR_OK)
				state_disk = CREATE_OK;
			else
				state_disk = CREATE_ERR;
			osDelay(1000);
		}
		else if (state_disk == LOGGING) //****   LOGGING ****
		{
			if(HAL_GetTick() >= period_complete) //    , :
			{
				period_complete += period*1000; //     +(period*1000ms)
				f_open(&LogFile, (char*)path, FA_WRITE | FA_OPEN_ALWAYS); //   
				f_lseek(&LogFile, LogFile.fsize); //     
				f_printf(&LogFile, "%02d.%02d.%02d %02d:%02d:%02d\t%d\n", //   :
				sDate.Date, sDate.Month, sDate.Year, 
				sTime.Hours, sTime.Minutes, sTime.Seconds, 
				av_temperature>>4); //   
				res = f_close(&LogFile); //  
				if (res == FR_OK)
					state_disk = SAVED_OK;
				else
					state_disk = SAVED_ERR;
			}
		}
		else //****   STOP ****
		{
			FATFS_UnLinkDriver(USER_Path);
		}
		Display(); //   ,  ,       
  }
  /* USER CODE END StartDefaultTask */
}

/* StartUSART function */
void StartUSART(void const * argument)
{
	uint8_t prompt[] = "\n\r>";
	uint8_t help[] = "\n\rSTART\n\rSTOP\n\rDATE dd mm yy\n\rTIME hh mm ss\n\rPERIOD hh mm ss\n\rDATE?\n\rTIME?\n\rPERIOD?\n\r";
	uint8_t err_cmd[] = "\n\rUnknown cmd - type HELP\n\r";
	uint8_t answer[9];
	
	int hd,m,sy; // , ,  / , , help
	char inpstr[8];
  /* USER CODE BEGIN StartUSART */
  /* Infinite loop */
  for(;;)
  {
		HAL_UART_Transmit(&huart1, prompt, sizeof(prompt), 100);
		scanf("%7s", inpstr); //     7 
		
		if (strcasecmp(inpstr,"HELP")==0)
		{
			HAL_UART_Transmit(&huart1, help, sizeof(help), 100); //    
		}
		else if(strcasecmp(inpstr,"START")==0)
		{
			state_disk = START;
		}
		else if(strcasecmp(inpstr,"STOP")==0)
		{
			state_disk = STOP;
		}
		else if(strcasecmp(inpstr,"DATE?")==0)
		{
			sprintf((char*)answer," %02d.%02d.%02d",sDate.Date,sDate.Month,sDate.Year);
			HAL_UART_Transmit(&huart1, answer, sizeof(answer), 100); //   
		}
		else if(strcasecmp(inpstr,"DATE")==0)
		{
			scanf("%i%i%i", &hd, &m, &sy); //  
			sDate.Date = hd;
			sDate.Month = m;
			sDate.Year = sy;
			HAL_RTC_SetDate(&hrtc, &sDate, RTC_FORMAT_BIN); //   
		}
		else if(strcasecmp(inpstr,"TIME?")==0)
		{
			sprintf((char*)answer," %02d:%02d:%02d",sTime.Hours,sTime.Minutes,sTime.Seconds);
			HAL_UART_Transmit(&huart1, answer, sizeof(answer), 100); //   
		}
		else if(strcasecmp(inpstr,"TIME")==0)
		{
			scanf("%i%i%i", &hd, &m, &sy); //  
			sTime.Hours = hd;
			sTime.Minutes = m;
			sTime.Seconds = sy;
			HAL_RTC_SetTime(&hrtc, &sTime, RTC_FORMAT_BIN); //   
		}
		else if(strcasecmp(inpstr,"PERIOD?")==0)
		{
			sy = (period%3600)%60;
			m = ((period-sy)/60)%60;
			hd = period/3600;
			sprintf((char*)answer," %02d:%02d:%02d",hd,m,sy);
			HAL_UART_Transmit(&huart1, answer, sizeof(answer), 100); //  
		}
		else if(strcasecmp(inpstr,"PERIOD")==0)
		{
			scanf("%i%i%i", &hd, &m, &sy); //  
			period = hd*3600 + m*60 + sy; //   
		}
		else
		{
			HAL_UART_Transmit(&huart1, err_cmd, sizeof(err_cmd), 100); //    
		}
    osDelay(99);
  }
  /* USER CODE END StartUSART */
}

/* USER CODE BEGIN Application */
     
/* USER CODE END Application */

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
